using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;

namespace WLog.Class
{
    /// <summary>
    /// Options of WLog
    /// </summary>
    [Serializable]
    public class Options
    {
        #region Variables
        /// <summary>
        /// Indicates whether the date is visible
        /// </summary>
        private bool vDate = true;
        /// <summary>
        /// Indicates whether the type of log is visible
        /// </summary>
        private bool vTypeLog = true;
        /// <summary>
        /// Indicates whether the message of log is visible
        /// </summary>
        private bool vMessage = true;
        /// <summary>
        /// Indicates whether the title of log is visible
        /// </summary>
        private bool vTitle = true;
        /// <summary>
        /// Indicates whether the menu is visible
        /// </summary>
        private bool vMenu = false;
        /// <summary>
        /// Gets the size of the menu
        /// </summary>
        public static readonly int SIZE_OF_MENU = 65;
        #endregion

        #region Constructors
        /// <summary>
        /// Constructor
        /// </summary>
        public Options() { }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the visibility of the log's date
        /// </summary>
        [Category("1- Visibility")]
        [Description("Sets the visibility of the log's date")]
        [DisplayName("1.1- Visibility of the date")]
        [DefaultValue(true)]
        public bool Date
        {
            get { return vDate; }
            set
            {
                vDate = value;
                OnOptionChanged();
            }
        }
        /// <summary>
        /// Gets or sets the visibility of the log's type
        /// </summary>
        [Category("1- Visibility")]
        [Description("Sets the visibility of the log's type")]
        [DisplayName("1.2- Visibility of the type")]
        [DefaultValue(true)]
        public bool TypeLog
        {
            get { return vTypeLog; }
            set
            {
                vTypeLog = value;
                OnOptionChanged();
            }
        }
        /// <summary>
        /// Gets or sets the visibility of log's title
        /// </summary>
        [Category("1- Visibility")]
        [Description("Sets the visibility of the log's title")]
        [DisplayName("1.3- Visibility of the title")]
        [DefaultValue(true)]
        public bool Title
        {
            get { return vTitle; }
            set
            {
                vTitle = value;
                OnOptionChanged();
            }
        }
        /// <summary>
        /// Gets or sets the visibility of log's message
        /// </summary>
        [Category("1- Visibility")]
        [Description("Sets the visibility of the log's message")]
        [DisplayName("1.4- Visibility of the message")]
        [DefaultValue(true)]
        public bool Message
        {
            get { return vMessage; }
            set
            {
                vMessage = value;
                OnOptionChanged();
            }
        }
        /// <summary>
        /// Gets or sets the visibility of the menu
        /// </summary>
        [Category("1- Menu")]
        [Description("Sets the visibility of the lomenu")]
        [DisplayName("2.1- Visibility of the menu")]
        [DefaultValue(false)]
        public bool Menu
        {
            get { return vMenu; }
            set
            {
                vMenu = value;
                OnOptionChanged();
            }
        }
        #endregion

        #region Events
        [NonSerialized]
        private EventHandler DelegateOptionChangedHandler;
        /// <summary>
        /// Thrown when options changes occures
        /// </summary>
        [Browsable(false)]
        public event EventHandler OptionChanged
        {
            add { DelegateOptionChangedHandler += value; }
            remove { DelegateOptionChangedHandler -= value; }
        }
        /// <summary>
        /// Notify of an option change
        /// </summary>                       
        protected virtual void OnOptionChanged()
        {
            if (DelegateOptionChangedHandler != null)
                DelegateOptionChangedHandler(this, new EventArgs());

        }
        #endregion
    }
}
